<?php
/**
* The box-object
*
* Provides functionality for drawing boxes around other objects
*
* @author	Alexander Wirtz <alex@pc4p.net>
* @package	pc4p
* @version	$Id: pc4p_box.inc,v 1.27 2002/08/12 11:09:58 eru Exp $
*/
class pc4p_box extends pc4p_object
{
	/**
	* Margins for the box
	*
	* @var		array	$margin
	* @access	public
	*/
	var $margin = array( "top" => 5, "bottom" => 5, "left" => 5, "right" => 5 );

	/**
	* Borders of the box
	*
	* @var		array	$borders
	* @access	public
	*/
	var $borders = array( "top" => NULL, "bottom" => NULL, "left" => NULL, "right" => NULL );

	/**
	* pc4p_box constructor
	*
	* Calls the constructor of pc4p_object
	*
	* @param	object	pc4p_page	&$parent	Parent of the box
	* @access	public
	* @see 		pc4p_init::&pc4p_create_object(), pc4p_object::pc4p_object(), pc4p_object::pc4p_set_rot()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_box( &$parent )
	{
		// Execute constructor of parent class
		pc4p_object::pc4p_object( $parent );

		// Create borders for the box
		$dummy = new pc4p_object( $this );
		foreach ( $this->borders as $side => $value ) {
			$this->borders[ $side ] = &pc4p_create_object( $dummy, "line" );
		}
		$this->borders[ "left" ]->pc4p_set_rot( -90 );
		$this->borders[ "right" ]->pc4p_set_rot( -90 );
	}

	/**
	* Set the style and width of the borders surrounding the box
	*
	* With this function you can set the borders of the box for each side individually, or completely
	*
	* @param	mixed	$border	"top", "bottom", "left", "right" or an array of these or "all"
	* @param	mixed	$style	"none", "single" or "double" or an array of these
	* @param	mixed	$width	The width of the line or an array of widths
	* @access	public
	* @see		pc4p_line::pc4p_set_linestyle(), pc4p_line::pc4p_set_linewidth()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_boxborder( $border = "all", $style = "single", $width = 1.0 ) {
		// First check which borders will get changed
		if ( !is_array( $border ) && in_array( $border, array( "top", "bottom", "left", "right" ) ) ) {
			$border = array( $border );
		}
		elseif ( $border == "all" ) {
			$border = array( "top", "bottom", "left", "right" );
		}
		// Then pad the style and width to arrays of same size as border, in case they aren't an array
		if ( !is_array( $style ) && in_array( $style, array( "none", "single", "double" ) ) ) {
			$style = array_pad( array(), sizeof( $border ), $style );
		}
		if ( !is_array( $width ) && is_numeric( $width ) ) {
			$width = array_pad( array(), sizeof( $border ), $width );
		}

		// Check for same size of border, style and width
		if ( sizeof( $border ) != sizeof( $style ) || sizeof( $border ) != sizeof( $width ) ) {
			die( "Error: ".get_class( $this )."(".__LINE__.") - when you set the borders of the box, the size of $border, $style and $width should be the same." );
		}

		// Then set the new parameters for the borders
		for ( $i = 0; $i < sizeof( $border ); $i++ ) {
			$this->borders[ $border[ $i ] ]->pc4p_set_linestyle( $style[ $i ] );
			$this->borders[ $border[ $i ] ]->pc4p_set_linewidth( $width[ $i ] );
		}
	}

	/**
	* Set the colors of the borders surrounding the box
	*
	* With this function you can set the colors of the borders of the box for each side individually, or completely
	*
	* @param	mixed	$border	"top", "bottom", "left", "right" or an array of these or "all"
	* @param	mixed	$color	You can use one of the 16 standardcolors from HTML or the hex-notation (#rrggbb) or an array of these
	* @access	public
	* @see		pc4p_line::pc4p_set_linecolor()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_boxbordercolor( $border = "all", $color = "" ) {
		// First check which borders will get changed
		if ( !is_array( $border ) && in_array( $border, array( "top", "bottom", "left", "right" ) ) ) {
			$border = array( $border );
		}
		elseif ( $border == "all" ) {
			$border = array( "top", "bottom", "left", "right" );
		}
		// Then pad the color to an array of same size as border, in case it isn't an array
		if ( !is_array( $color ) ) {
			$color = array_pad( array(), sizeof( $border ), $color );
		}

		// Check for same size of border and color
		if ( sizeof( $border ) != sizeof( $color ) ) {
			die( "Error: ".get_class( $this )."(".__LINE__.") - when you set the bordercolors of the cell, the size of \$border and \$color should be the same." );
		}

		// Then set the new color for the borders
		for ( $i = 0; $i < sizeof( $border ); $i++ ) {
			$this->borders[ $border[ $i ] ]->pc4p_set_linecolor( $color[ $i ] );
		}
	}

	/**
	* Draw-function for the box
	*
	* Calls the pc4p_draw in its children, then draws the box
	*
	* @access	private
	* @see		pc4p_line::pc4p_draw(), pc4p_box::pc4p_draw_children()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_draw()
	{
		// Check, if the box is having a fillcolor, if yes, create a filled rectangle under the box
		if ( is_array( $this->fillcolor ) ) {
			pdf_save( $this->pdfp );
			pdf_setcolor( $this->pdfp, "fill", "rgb", $this->fillcolor[ 0 ], $this->fillcolor[ 1 ], $this->fillcolor[ 2 ], 0 );
			pdf_rect( $this->pdfp, $this->act_x + $this->margin[ "left" ], -( $this->act_y + $this->margin[ "top" ] ), $this->width - $this->margin[ "left" ] - $this->margin[ "right" ], -( $this->height  - $this->margin[ "top" ] - $this->margin[ "bottom" ] ) );
			pdf_fill( $this->pdfp );
			pdf_restore( $this->pdfp );
		}

		// Draw the borders of the box...
		foreach ( $this->borders as $side => $values ) {
			$this->borders[ $side ]->pc4p_draw();
		}

		// ...and draw the children within it
		$this->pc4p_draw_children();

		unset( $this->pdfobj );
	}

	/**
	* Calls the calc_offset function in all children, sets its own offsets
	*
	* Checks for a already set width, watches for alignments
	*
	* @param	object	pc4p_page	&$parent	Parent of the object
	* @return	integer	$this->height	Calculated height of the object
	* @access	private
	* @see		pc4p_line::pc4p_get_linewidth(), pc4p_line::pc4p_calc_linejoin(), pc4p_line::pc4p_calc_offset(), pc4p_line::pc4p_set_width()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_calc_offset( &$parent )
	{
		if ( get_class( $parent ) == "pc4p_page" ) {
			$this->pagewidth = $parent->width;
			$this->pageheight = $parent->height;
		}
		else {
			$this->pagewidth = $parent->pagewidth;
			$this->pageheight = $parent->pageheight;
		}

		// Check, if someone set the width externally and as a relative value
		if ( isset( $this->width ) && preg_match( "/^(\d+)%$/", $this->width, $width ) ) {
			// Yes, someone did. So erase the % and calculate the width
			$this->width = ( $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] ) ) * $width[ 1 ] / 100.0;
		}
		// Check, if width is set, and if yes, if the width exceeds max width defined by parent
		if ( !isset( $this->width ) || $this->width <= 0 || $this->width > $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] ) ) {
			// If-Clause is true, so set width to max width
			$this->width = $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] );
		}

		// Add borderwidths to margins to make sure, the children of the box don't interfere with the borders
		foreach ( $this->margin as $side => $value ) {
			$this->margin[ $side ] += $this->borders[ $side ]->pc4p_get_linewidth();
		}

		$this->height = $this->margin[ "top" ];
		$this->act_x = $parent->draw_x;
		$this->act_y = $parent->draw_y;

		// Calculate starting x-coordinate dependant on the alignment
		if ( $this->alignment == "center" ) {
			$this->draw_x = $this->act_x + $this->margin[ "left" ] + ( $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] ) - $this->width ) / 2;
		}
		elseif ( $this->alignment == "right" ) {
			$this->draw_x = $this->act_x + $this->margin[ "left" ] + $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] ) - $this->width;
		}
		else {
			$this->draw_x = $this->act_x + $this->margin[ "left" ];
		}

		$this->draw_y = $this->act_y + $this->margin[ "top" ];
		// Calculate the heights in the children and add them to our height
		for ( $i = 0; $i < sizeof( $this->children ); $i++) {
			$child_height = $this->children[$i]->pc4p_calc_offset( $this );
			$this->height += $child_height;
			$this->draw_y += $child_height;
		}
		$this->draw_y += $this->margin[ "bottom" ];
		$this->height += $this->margin[ "bottom" ];

		// Allocate dummy object to serve as parent for the calculations of the borders
		$dummy = new pc4p_object( $this );
		$dummy->pagewidth = $this->pagewidth;
		$dummy->pageheight = $this->pageheight;
		$dummy->height = $this->height - ( $this->margin[ "top" ] + $this->margin[ "bottom" ] ) + ( $this->borders[ "top" ]->pc4p_get_linewidth() + $this->borders[ "bottom" ]->pc4p_get_linewidth() );
		$dummy->width = $this->width - ( $this->margin[ "left" ] + $this->margin[ "right" ] ) + ( $this->borders[ "left" ]->pc4p_get_linewidth() + $this->borders[ "right" ]->pc4p_get_linewidth() );

		// Now set the grid coordinates and dimensions for each border
		$dummy->draw_x = $this->draw_x - $this->borders[ "left" ]->pc4p_get_linewidth();
		$dummy->draw_y = $parent->draw_y + $this->margin[ "top" ] - $this->borders[ "top" ]->pc4p_get_linewidth();
		$this->borders[ "top" ]->pc4p_set_width( $dummy->width );
		$this->borders[ "top" ]->pc4p_calc_offset( $dummy );

		$dummy->draw_x = $this->draw_x - $this->borders[ "left" ]->pc4p_get_linewidth();
		$dummy->draw_y = $parent->draw_y + $this->margin[ "top" ] - $this->borders[ "top" ]->pc4p_get_linewidth() - $this->borders[ "bottom" ]->pc4p_get_linewidth() + $dummy->height;
		$this->borders[ "bottom" ]->pc4p_set_width( $dummy->width );
		$this->borders[ "bottom" ]->pc4p_calc_offset( $dummy );

		$dummy->draw_x = $this->draw_x - $this->borders[ "left" ]->pc4p_get_linewidth();
		$dummy->draw_y = $parent->draw_y + $this->margin[ "top" ] - $this->borders[ "top" ]->pc4p_get_linewidth();
		$this->borders[ "left" ]->pc4p_set_width( $dummy->height );
		$this->borders[ "left" ]->pc4p_calc_offset( $dummy );

		$dummy->draw_x = $this->draw_x - $this->borders[ "left" ]->pc4p_get_linewidth() - $this->borders[ "right" ]->pc4p_get_linewidth() + $dummy->width;
		$dummy->draw_y = $parent->draw_y + $this->margin[ "top" ] - $this->borders[ "top" ]->pc4p_get_linewidth();
		$this->borders[ "right" ]->pc4p_set_width( $dummy->height );
		$this->borders[ "right" ]->pc4p_calc_offset( $dummy );

		// To have decent linejoins
		$d = 0;
		pc4p_line::pc4p_calc_linejoin( $this->borders[ "top" ], $this->borders[ "left" ], $d, $d );
		pc4p_line::pc4p_calc_linejoin( $d, $this->borders[ "right" ], $this->borders[ "top" ], $d );
		pc4p_line::pc4p_calc_linejoin( $this->borders[ "bottom" ], $d, $d, $this->borders[ "left" ] );
		pc4p_line::pc4p_calc_linejoin( $d, $d, $this->borders[ "bottom" ], $this->borders[ "right" ] );

		return $this->height;
	}
}
?>
