<?php
/**
* PC4P's init module
*
* This is the file, you _must_ include. It sets the needed include path and provides wrapper functions for the objects. 
*
* @module	pc4p_init
* @modulegroup	pc4p
* @author	Alexander Wirtz <alex@pc4p.net>
* @package	pc4p
* @version	$Id: pc4p_init.inc,v 1.38 2002/08/12 10:50:39 eru Exp $
*/

// Check for a few versions. If they're not met, die with grace
if ( !extension_loaded( "pdf" ) ) {
	die( "Please install a PDFLib !" );
}

if ( !preg_match( "/(4\.0\.([5-9]|\d{2,}))|(4\.([1-9]|\d{2,})\.\d+)|(([5-9]|\d{2,})\.\d+\.\d+).*/", phpversion() ) ) {
	die( "Please install a PHP >= 4.0.5 !" );
}

// Set the include path
$include_path = dirname( __FILE__ );
$ini_include_path = ini_get( "include_path" );
$ini_include_path .= !strpos( $ini_include_path, ";" ) ? ":" . $include_path : ";" . $include_path;
ini_set( "include_path", $ini_include_path );

// Include the classes
if ( !defined( "PC4PINCLUDES" ) ) {
	/**
	* Include the main object
	*
	* @include	pc4p_main.inc
	*/
	require_once( "pc4p_main.inc" );

	/**
	* Include the page object
	*
	* @include	pc4p_page.inc
	*/
	require_once( "pc4p_page.inc" );

	/**
	* Include the generic object
	*
	* @include	pc4p_object.inc
	*/
	require_once( "pc4p_object.inc" );

	/**
	* Include the object for using text
	*
	* @include	pc4p_text.inc
	*/
	require_once( "pc4p_text.inc" );

	/**
	* Include the object for drawing boxes
	*
	* @include	pc4p_box.inc
	*/
	require_once( "pc4p_box.inc" );

	/**
	* Include the object for drawing boxes
	*
	* @include	pc4p_line.inc
	*/
	require_once( "pc4p_line.inc" );

	/**
	* Include the object for creating tables
	*
	* @include	pc4p_table.inc
	*/
	require_once( "pc4p_table.inc" );

	/**
	* Include the object for images
	*
	* @include pc4p_image
	*/
	require_once( "pc4p_image.inc" );

	/**
	* Include the object for pagenumbers
	*
	* @include pc4p_pagenumber
	*/
	require_once( "pc4p_pagenumber.inc" );

	/**
	* Is defined, if the script has included the subclasses
	*
	* @const PC4PINCLUDES
	*/
	define( "PC4PINCLUDES", TRUE );
}

	/**
	* pc4p_create_pdf - Main wrapper function to create the PDF
	*
	* This is the first function which should be called, as it acquires the PDF-Pointer for all other operations.
	*
	* @param	array	$info					here go the vars for the infos of the PDF
	* @return	object	pc4p_main	$pdf		the pdf-object, used by pc4p_create_page
	* @access	public
	* @see		pc4p_main::pc4p_main(), pc4p_main::pc4p_set_info()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function &pc4p_create_pdf( $info = array() )
	{
		$pdf = &new pc4p_main();
		if ( !empty( $info ) ) {
			$pdf->pc4p_set_info( $info );
		}
		return $pdf;
	}

	/**
	* pc4p_create_page - Wrapper function to create a page
	*
	* This function is used to create new pages. Can take a standard pagesize or user defines.
	*
	* @param	object	pc4p_main	&$parent	parent of the page, can only be a PDF-Object
	* @param	string	$format					name of a standard pagesize or "user"
	* @param	string	$orientation			"portrait", "p" or "landscape", "l" - will be ignored, if format is "user"
	* @param	string	$dimensions				or the dimensions if page is "user"-defined <width>x<height>
	* @return	object	pc4p_page	$page		a page-object is returned, used by pc4p_create_object
	* @access	public
	* @see		pc4p_page::pc4p_page()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function &pc4p_create_page( &$parent, $format, $orientation = "p", $dimensions = "" )
	{
		$page = &new pc4p_page( $parent, $format, $orientation, $dimensions );
		$parent->children[] = &$page;
		return $page;
	}

	/**
	* pc4p_create_object - Wrapper function to create objects
	*
	* Here we have the wrapper for all the other objects: Text, tables, boxes, images and a object, which is used as container.
	*
	* @param	object	pc4p_page	&$parent	parent of the object, usually a page, or some container-object such as boxes and tables
	* @param	string	$object_type			type of object to be created, if empty a container-object is created
	* @return	object	pc4p_object	$object		the object, which was created by this function
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function &pc4p_create_object( &$parent, $object_type = "object" )
	{
		// Test for the object-type, and create a new object, if
		// type is valid, bail out, if it's not
		$class = "pc4p_".$object_type;
		if ( !class_exists( $class ) ) {
			die( "Error: pc4p_init(".__LINE__.") - ".$object_type." is not a valid object-type." );
		}
		$object = &new $class( $parent );

		// Move a reference into the parents array...
		$parent->children[] = &$object;
		// and return the created object
		return $object;
	}
?>
