<?php
/**
* The line-object
*
* Provides functionality for drawing lines
*
* @author	Alexander Wirtz <alex@pc4p.net>
* @package	pc4p
* @version	$Id: pc4p_line.inc,v 1.23 2002/08/12 10:50:39 eru Exp $
*/
class pc4p_line extends pc4p_object
{
	/**
	* Margins for the line
	*
	* @var		array	$margin
	* @access	public
	*/
	var $margin = array( "top" => 0, "bottom" => 0, "left" => 0, "right" => 0 );

	/**
	* Style of the line
	*
	* @var		string	$linestyle
	* @access	public
	*/
	var $linestyle = "single";

	/**
	* Width of the line
	*
	* @var		float	$linewidth
	* @access	public
	*/
	var $linewidth = 1.0;

	/**
	* Factors to correct the lines when they meet
	*
	* @var		array	$linemeet
	* @access   private
	*/
	var $linemeet = array( 0, 0, 0, 0 );

	/**
	* pc4p_line constructor
	*
	* Calls the constructor of pc4p_object
	*
	* @param	object	pc4p_page	&$parent	Parent of the line
	* @access	public
	* @see		pc4p_object::pc4p_object()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_line( &$parent )
	{
		pc4p_object::pc4p_object( $parent );
	}

	/**
	* Sets the style for the line
	*
	* Well, it lets you define, what line you want...
	*
	* @param	string	$style	Valid values are "none", "single" or "double"
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_linestyle( $style )
	{
		$this->linestyle = "single";
		if ( $style == "double" ) {
			$this->linestyle = $style;
		}
		elseif ( $style == "none" ) {
			$this->linestyle = $style;
		}
	}

	/**
	* Sets the width of the line
	*
	* Guess what, with that func you define the linewidth!
	*
	* @param	float	$width	Values >= 0 are accepted
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_linewidth( $width )
	{
		if ( $width >= 0 ) {
			$this->linewidth = $width;
		}
	}

	/**
	* Returns the total (!) width of the line
	*
	* This function returns the width according to the style
	*
	* @return	float		$linewidth	Calculated total width of the line
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_get_linewidth()
	{
		if ( $this->linestyle == "double" ) {
			return 2 * $this->linewidth + 2 * $this->linewidth / 3.0;
		}
		elseif ( $this->linestyle == "none" ) {
			return 0.0;
		}
		else {
			return (float) $this->linewidth;
		}
	}

	/**
	* Calculate the ends of the lines
	*
	* When the lines meet in a corner, their ends have to be adjusted, so that everything fits smugly together. Well, this function handles that task...and it's no easy task at that. This function is ugly, if someone is capable of beautifying this, please send me an email...
	*
   	* @param	object	pc4p_line	&$h_line1	Upper horizontal line
	* @param	object	pc4p_line	&$v_line1	Left vertical line
	* @param	object	pc4p_line	&$h_line2	Lower horizontal line
	* @param	object	pc4p_line	&$v_line2	Right vertical line
	* @access	private
	* @see		pc4p_line::pc4p_get_linewidth()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_calc_linejoin( &$h_line1, &$v_line1, &$h_line2, &$v_line2 )
	{
		// I won't comment this stuff here very much, it is almost the same everytime
		// First determine how many corners we have, than compare styles of the neighbours
		// and so on...took me hours :-P
		if ( is_object( $h_line1 ) && is_object( $v_line1 ) && is_object( $h_line2 ) && is_object( $v_line2 ) ) {
			if ( $h_line1->linestyle == "single" ) {
				if ( ( $v_line1->linestyle == "double" && $v_line2->linestyle == "double" ) || 
					( $h_line2->linestyle == "double" && ( $v_line1->linestyle == "double" || $v_line2->linestyle == "double" ) ) ) {
					$h_line1->linemeet[ 0 ] =  min( $v_line1->linewidth, $v_line2->linewidth ) / 3.0;
					$h_line1->linemeet[ 2 ] =  0;
				}
			}
			else {
				if ( $v_line1->linestyle == "double" || $v_line2->linestyle == "double" || $h_line2->linestyle == "double" ) {
					$h_line1->linemeet[ 0 ] =  $v_line2->pc4p_get_linewidth() - 2 * $v_line2->linewidth - $v_line2->linewidth / 3.0;
					$h_line1->linemeet[ 2 ] =  $v_line1->pc4p_get_linewidth() - 2 * $v_line1->linewidth - $v_line1->linewidth / 3.0;
				}
				else {
					$h_line1->linemeet[ 0 ] =  0;
					$h_line1->linemeet[ 2 ] =  0;
				}
			}
			if ( $v_line1->linestyle == "single" ) {
				if ( ( $h_line1->linestyle == "double" && $h_line2->linestyle == "double" ) ||
					( $v_line2->linestyle == "double" && ( $h_line1->linestyle == "double" || $h_line2->linestyle == "double" ) ) ) {
					$v_line1->linemeet[ 0 ] =  min( $h_line1->linewidth, $h_line2->linewidth ) / 3.0;
					$v_line1->linemeet[ 2 ] =  0;
				}
			}
			else {
				if ( $h_line1->linestyle == "double" || $h_line2->linestyle == "double" || $v_line2 == "double" ) {
					$v_line1->linemeet[ 0 ] =  $h_line1->pc4p_get_linewidth() - 2 * $h_line1->linewidth - $h_line1->linewidth / 3.0;
					$v_line1->linemeet[ 2 ] =  $h_line2->pc4p_get_linewidth() - 2 * $h_line2->linewidth - $h_line2->linewidth / 3.0;
				}
				else {
					$v_line1->linemeet[ 0 ] =  0;
					$v_line1->linemeet[ 2 ] =  0;
				}
			}
			if ( $h_line2->linestyle == "single" ) {
				if ( ( $v_line1->linestyle == "double" && $v_line2->linestyle == "double" ) || 
					( $h_line1->linestyle == "double" && ( $v_line1->linestyle == "double" || $v_line2->linestyle == "double" ) ) ) {
					$h_line2->linemeet[ 1 ] = -min( $v_line1->linewidth, $v_line2->linewidth ) / 3.0;
					$h_line2->linemeet[ 3 ] =  0;
				}
			}
			else {
				if ( $v_line1->linestyle == "double" || $v_line2->linestyle == "double" || $h_line1->linestyle == "double" ) {
					$h_line2->linemeet[ 1 ] = -$v_line2->pc4p_get_linewidth() + 2 * $v_line2->linewidth + $v_line2->linewidth / 3.0;
					$h_line2->linemeet[ 3 ] = -$v_line1->pc4p_get_linewidth() + 2 * $v_line1->linewidth + $v_line1->linewidth / 3.0;
				}
				else {
					$h_line2->linemeet[ 1 ] =  0;
					$h_line2->linemeet[ 3 ] =  0;
				}
			}
			if ( $v_line2->linestyle == "single" ) {
				if ( ( $h_line1->linestyle == "double" && $h_line2->linestyle == "double" ) ||
					( $v_line1->linestyle == "double" && ( $h_line1->linestyle == "double" || $h_line2->linestyle == "double" ) ) ) {
					$v_line2->linemeet[ 1 ] = -min( $h_line1->linewidth, $h_line2->linewidth ) / 3.0;
					$v_line2->linemeet[ 3 ] =  0;
				}
			}
			else {
				if ( $h_line1->linestyle == "double" || $h_line2->linestyle == "double" || $v_line1 == "double" ) {
					$v_line2->linemeet[ 1 ] = -$h_line1->pc4p_get_linewidth() + 2 * $h_line1->linewidth + $h_line1->linewidth / 3.0;
					$v_line2->linemeet[ 3 ] = -$h_line2->pc4p_get_linewidth() + 2 * $h_line2->linewidth + $h_line2->linewidth / 3.0;
				}
				else {
					$v_line2->linemeet[ 1 ] =  0;
					$v_line2->linemeet[ 3 ] =  0;
				}
			}
		}
		elseif ( is_object( $h_line1 ) && is_object( $v_line1 ) && is_object( $h_line2 ) ) {
			if ( $h_line1->linestyle == "double" && $v_line1->linestyle == "double" ) {
				$h_line1->linemeet[ 0 ] = -$v_line1->linewidth / 3.0;
				$h_line1->linemeet[ 2 ] =  $v_line1->pc4p_get_linewidth() - $v_line1->linewidth - $v_line1->linewidth / 3.0;
			}
			if ( ( $v_line1->linestyle == "double" && $h_line1->linestyle == "double" ) ||
				( $v_line1->linestyle == "single" && $h_line1->linestyle == "double" && $h_line2->linestyle == "double" ) ) {
				$v_line1->linemeet[ 0 ] =  $h_line1->pc4p_get_linewidth() - $h_line1->linewidth;
			}
			if ( $v_line1->linestyle == "double" && $h_line2->linestyle == "double" ) {
				$v_line1->linemeet[ 2 ] =  $h_line2->pc4p_get_linewidth() - $h_line2->linewidth;
			}
			if ( $h_line2->linestyle == "double" && $v_line1->linestyle == "double" ) {
				$h_line2->linemeet[ 1 ] =  $v_line1->linewidth / 3.0;
				$h_line2->linemeet[ 3 ] = -$v_line1->pc4p_get_linewidth() + $v_line1->linewidth + $v_line1->linewidth / 3.0;
			}
		}
		elseif ( is_object( $h_line1 ) && is_object( $v_line1 ) && is_object( $v_line2 ) ) {
			if ( ( $h_line1->linestyle == "double" && $v_line2->linestyle == "double" ) ||
				( $h_line1->linestyle == "single" && $v_line1->linestyle == "double" && $v_line2->linestyle == "double" ) ) {
				$h_line1->linemeet[ 0 ] =  $v_line2->pc4p_get_linewidth() - $v_line2->linewidth;
			}
			if ( $h_line1->linestyle == "double" && $v_line1->linestyle == "double" ) {
				$h_line1->linemeet[ 2 ] =  $v_line1->pc4p_get_linewidth() - $v_line1->linewidth;
			}
			if ( $v_line1->linestyle == "double" && $h_line1->linestyle == "double" ) {
				$v_line1->linemeet[ 0 ] =  $h_line1->pc4p_get_linewidth() - $h_line1->linewidth - $h_line1->linewidth / 3.0;
				$v_line1->linemeet[ 2 ] = -$h_line1->linewidth / 3.0;
			}
			if ( $v_line2->linestyle == "double" && $h_line1->linestyle == "double" ) {
				$v_line2->linemeet[ 1 ] = -$h_line1->pc4p_get_linewidth() + $h_line1->linewidth + $h_line1->linewidth / 3.0;
				$v_line2->linemeet[ 3 ] =  $h_line1->linewidth / 3.0;
			}
		}
		elseif ( is_object( $v_line1 ) && is_object( $h_line2 ) && is_object( $v_line2 ) ) {
			if ( $v_line1->linestyle == "double" && $h_line2->linestyle == "double" ) {
				$v_line1->linemeet[ 0 ] = -$h_line2->linewidth / 3.0;
				$v_line1->linemeet[ 2 ] =  $h_line2->pc4p_get_linewidth() - $h_line2->linewidth - $h_line2->linewidth / 3.0;
			}
			if ( ( $h_line2->linestyle == "double" && $v_line2->linestyle == "double" ) ||
				( $h_line2->linestyle == "single" && $v_line1->linestyle == "double" && $v_line2->linestyle == "double" ) ) {
				$h_line2->linemeet[ 1 ] = -$v_line2->pc4p_get_linewidth() + $v_line2->linewidth;
			}
			if ( $h_line2->linestyle == "double" && $v_line1->linestyle == "double" ) {
				$h_line2->linemeet[ 3 ] = -$v_line1->pc4p_get_linewidth() + $v_line1->linewidth;
			}
			if ( $v_line2->linestyle == "double" && $h_line2->linestyle == "double" ) {
				$v_line2->linemeet[ 1 ] =  $h_line2->linewidth / 3.0;
				$v_line2->linemeet[ 3 ] = -$h_line2->pc4p_get_linewidth() + $h_line2->linewidth + $h_line2->linewidth / 3.0;
			}
		}
		elseif ( is_object( $h_line1 ) && is_object( $h_line2 ) && is_object( $v_line2 ) ) {
			if ( $h_line1->linestyle == "double" && $v_line2->linestyle == "double" ) {
				$h_line1->linemeet[ 0 ] =  $v_line2->pc4p_get_linewidth() - $v_line2->linewidth - $v_line2->linewidth / 3.0;
				$h_line1->linemeet[ 2 ] = -$v_line2->linewidth / 3.0;
			}
			if ( $h_line2->linestyle == "double" && $v_line2->linestyle == "double" ) {
				$h_line2->linemeet[ 1 ] = -$v_line2->pc4p_get_linewidth() + $v_line2->linewidth + $v_line2->linewidth / 3.0;
				$h_line2->linemeet[ 3 ] =  $v_line2->linewidth / 3.0;
			}
			if ( ( $v_line2->linestyle == "double" && $h_line1->linestyle == "double" ) ||
				( $v_line2->linestyle == "single" && $h_line1->linestyle == "double" && $h_line2->linestyle == "double" ) ) {
				$v_line2->linemeet[ 1 ] = -$h_line1->pc4p_get_linewidth() + $h_line1->linewidth;
			}
			if ( $v_line2->linestyle == "double" && $h_line2->linestyle == "double" ) {
				$v_line2->linemeet[ 3 ] = -$h_line2->pc4p_get_linewidth() + $h_line2->linewidth;
			}
		}
		elseif ( is_object( $h_line1 ) && is_object( $v_line1 ) ) {
			if ( $h_line1->linestyle == "double" && $v_line1->linestyle == "double" ) {
				$h_line1->linemeet[ 0 ] =  0;
				$h_line1->linemeet[ 2 ] =  $v_line1->pc4p_get_linewidth() - $v_line1->linewidth;
				$v_line1->linemeet[ 0 ] =  $h_line1->pc4p_get_linewidth() - $h_line1->linewidth;
				$v_line1->linemeet[ 2 ] =  0;
			}
		}
		elseif ( is_object( $h_line1 ) && is_object( $v_line2 ) ) {
			if ( $h_line1->linestyle == "double" && $v_line2->linestyle == "double" ) {
				$h_line1->linemeet[ 0 ] =  $v_line2->pc4p_get_linewidth() - $v_line2->linewidth;
				$h_line1->linemeet[ 2 ] =  0;
				$v_line2->linemeet[ 1 ] = -$h_line1->pc4p_get_linewidth() + $h_line1->linewidth;
				$v_line2->linemeet[ 3 ] =  0;
			}
		}
		elseif ( is_object( $h_line2 ) && is_object( $v_line1 ) ) {
			if ( $h_line2->linestyle == "double" && $v_line1->linestyle == "double" ) {
				$h_line2->linemeet[ 1 ] =  0;
				$h_line2->linemeet[ 3 ] = -$v_line1->pc4p_get_linewidth() + $v_line1->linewidth;
				$v_line1->linemeet[ 0 ] =  0;
				$v_line1->linemeet[ 2 ] =  $h_line2->pc4p_get_linewidth() - $h_line2->linewidth;
			}
		}
		elseif ( is_object( $h_line2 ) && is_object( $v_line2 ) ) {
			if ( $h_line2->linestyle == "double" && $v_line2->linestyle == "double" ) {
				$h_line2->linemeet[ 1 ] = -$v_line2->pc4p_get_linewidth() + $v_line2->linewidth;
				$h_line2->linemeet[ 3 ] =  0;
				$v_line2->linemeet[ 1 ] =  0;
				$v_line2->linemeet[ 3 ] = -$h_line2->pc4p_get_linewidth() + $h_line2->linewidth;
			}
		}
	}

	/**
	* Draw-function for the line
	*
	* Draws the line according to the linestyle
	*
	* @access	private
	* @see		pc4p_object::pc4p_calc_rot(), pc4p_object::pc4p_calc_rot_dim()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_draw()
	{
		if ( $this->linestyle == "none" ) {
			unset( $this->pdfobj );
			return 0;
		}

		pdf_save( $this->pdfp );
		pdf_setcolor( $this->pdfp, "stroke", "rgb", $this->linecolor[ 0 ], $this->linecolor[ 1 ], $this->linecolor[ 2 ], 0 );
		pdf_setcolor( $this->pdfp, "fill", "rgb", $this->linecolor[ 0 ], $this->linecolor[ 1 ], $this->linecolor[ 2 ], 0 );
		pdf_setlinewidth( $this->pdfp, $this->linewidth );

		// Calculate rotation
		$rot_dim = $this->pc4p_calc_rot_dim( $this->width, $this->height, $this->phi );
		$transp  = $this->pc4p_calc_rot( $this->draw_x + $rot_dim[ "width" ] / 2.0, -( $this->draw_y + $rot_dim[ "height" ] / 2.0 ), -$this->phi );
		pdf_rotate( $this->pdfp, $this->phi );

		if ( $this->linestyle == "double" ) {
			pdf_moveto( $this->pdfp, $transp[ "x" ] - $this->width / 2.0 + $this->linemeet[ 0 ], $transp[ "y" ] + $this->linewidth / 2.0 + $this->linewidth / 3.0 );
			pdf_lineto( $this->pdfp, $transp[ "x" ] + $this->width / 2.0 + $this->linemeet[ 1 ], $transp[ "y" ] + $this->linewidth / 2.0 + $this->linewidth / 3.0 );
			pdf_moveto( $this->pdfp, $transp[ "x" ] - $this->width / 2.0 + $this->linemeet[ 2 ], $transp[ "y" ] - $this->linewidth / 2.0 - $this->linewidth / 3.0 );
			pdf_lineto( $this->pdfp, $transp[ "x" ] + $this->width / 2.0 + $this->linemeet[ 3 ], $transp[ "y" ] - $this->linewidth / 2.0 - $this->linewidth / 3.0 );
		}
		else {
			pdf_moveto( $this->pdfp, $transp[ "x" ] - $this->width / 2.0 + $this->linemeet[ 0 ], $transp[ "y" ] );
			pdf_lineto( $this->pdfp, $transp[ "x" ] + $this->width / 2.0 + $this->linemeet[ 1 ], $transp[ "y" ] );
		}
		pdf_closepath_stroke( $this->pdfp );
		pdf_restore( $this->pdfp );

		unset( $this->pdfobj );
	}

	/**
	* Calculates the dimensions for the line
	*
	* Sets its own offsets according to linestyle...nothing spectacular
	*
	* @param	object	pc4p_page	&$parent	Parent of the image
	* @return	integer		$this->height	Calculated height of the line
	* @access	private
	* @see		pc4p_object::pc4p_calc_rot_dim()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_calc_offset( &$parent )
	{
		if ( $this->linestyle == "none" ) {
			$this->linewidth = 0.0;
			return 0;
		}

		if ( get_class( $parent ) == "pc4p_page" ) {
			$this->pagewidth = $parent->width;
			$this->pageheight = $parent->height;
		}
		else {
			$this->pagewidth = $parent->pagewidth;
			$this->pageheight = $parent->pageheight;
		}

		// Check, if someone set the width externally and as a relative value
		if ( isset( $this->width ) && preg_match( "/^(\d+)%$/", $this->width, $width ) ) {
			// Yes, someone did. So erase the % and calculate the width
			$this->width = ( $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] ) ) * $width[ 1 ] / 100.0;
		}
		// Check, if width is set, and if yes, if the width exceeds max width defined by parent
		if ( !isset( $this->width ) || $this->width < 0 ) {
			// If-Clause is true, so set width to max width
			$this->width = $parent->width - ( $this->margin[ "left" ] + $this->margin[ "right" ] + $parent->margin[ "left" ] + $parent->margin[ "right" ] );
		}

		if ( $this->linestyle == "double" ) {
			$this->height = 2 * $this->linewidth + 2 * $this->linewidth / 3.0;
		}
		else {
			$this->height = $this->linewidth;
		}

		$rot_dim = $this->pc4p_calc_rot_dim( $this->width, $this->height, $this->phi );

		if ( $rot_dim[ "width" ] > $parent->width - ( $this->margin[ "left" ] + $this->margin[ "right" ] + $parent->margin[ "left" ] + $parent->margin[ "right" ] ) ) {
			$this->width *= ( $parent->width - ( $this->margin[ "left" ] + $this->margin[ "right" ] + $parent->margin[ "left" ] + $parent->margin[ "right" ] ) ) / $rot_dim[ "width" ];
			$rot_dim = $this->pc4p_calc_rot_dim( $this->width, $this->height, $this->phi );
		}

		$this->act_x = $parent->draw_x;
		$this->act_y = $parent->draw_y;

		if ( $this->alignment == "center" ) {
			$this->draw_x = $this->act_x + ( $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] + $this->margin[ "left" ] + $this->margin[ "right" ] ) - $rot_dim[ "width" ] ) / 2.0;
		}
		elseif ( $this->alignment == "right" ) {
			$this->draw_x = $this->act_x + $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] + $this->margin[ "left" ] + $this->margin[ "right" ] ) - $rot_dim[ "width" ];
		}
		else {
			$this->draw_x = $this->act_x + $this->margin[ "left" ];
		}

		$this->draw_y = $this->act_y + $this->margin[ "top" ];

		return $this->margin[ "top" ] + $rot_dim[ "height" ]+ $this->margin[ "bottom" ];
	}
}
?>
