<?php
/**
* The table-object
*
* Provides table-functionality
*
* @author	Alexander Wirtz <alex@pc4p.net>
* @package	pc4p
* @version	$Id: pc4p_table.inc,v 1.59 2002/08/12 11:09:58 eru Exp $
*/
class pc4p_table extends pc4p_object
{
	/**
	* Array for accessing the cells of the table
	*
	* @var		array	$cell
	* @access	private
	*/
	var $cell;

	/**
	* Array containing the sizes of each column
	*
	* @var		array	$colsize
	* @access	private
	*/
	var $colsize;

	/**
	* Array containing the sizes of each row
	*
	* @var		array	$rowsize
	* @access	private
	*/
	var $rowsize;

	/**
	* Array containing the names of each column
	*
	* @var		array	$colnames
	* @access	private
	*/
	var $colnames = array();

	/**
	* Array containing the names of each row
	*
	* @var		array	$rownames
	* @access	private
	*/
	var $rownames = array();

	/**
	* Contains the name of the row, where the header of this
	* table is in
	*
	* @var		string	$tableheader
	* @access	private
	*/
	var $tableheader;

	/**
	* pc4p_table constructor
	*
	* Calls the constructor of pc4p_object
	*
	* @param	object	pc4p_page	&$parent	Parent of the table
	* @access	public
	* @see		pc4p_object::pc4p_object()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_table( &$parent )
	{
		pc4p_object::pc4p_object( $parent );
	}

	/**
	* Creates a matrix in the table-object 
	*
	* Creates the table and sets the names for columns and rows if provided, otherwise an index is used as name for each.
	*
	* @param	integer	$colsize	Number of columns
	* @param	integer	$rowsize	Number of rows
	* @param	array	$colnames	Names of the columns - if provided, size of the array has to correspond with the number of columns
	* @param	array	$rownames	Names of the rows - if provided, size of the array has to correspond with the number of rows
	* @access	public
	* @see		pc4p_init::&pc4p_create_object()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_create_tablematrix( $colsize, $rowsize = 1, $colnames = array(), $rownames = array() )
	{
		// Check if columnnames are provided
		if ( !empty( $colnames ) ) {
			// if yes, are there enough for all cols?
			if ( sizeof( $colnames ) != $colsize ) {
				die( "Error: ".get_class( $this )."(".__LINE__.") - size of columnnames is not correct or columnames are not unique." );
			}
		}
		else {
			// there are no names, so use an index
			for ( $i = 0; $i < $colsize; $i++ ) {
				$colnames[] = $i;
			}
		}
		// set the colnames for this object
		$this->colnames = $colnames;

		// Check if rownames are provided
		if ( !empty( $rownames ) ) {
			// if yes, are there enough for all rows?
			if ( sizeof( $rownames ) != $rowsize ) {
				die( "Error: ".get_class( $this )."(".__LINE__.") - size of rownames is not correct or rownames are not unique." );
			}
		}
		else {
			// there are no names, so use an index
			for ( $i = 0; $i < $rowsize; $i++ ) {
				$rownames[] = $i;
			}
		}
		// set the rownames for this object
		$this->rownames = $rownames;

		// use this dummy as object for anchoring the cellobjects and the borders
		$dummy = new pc4p_object( $this );
		for ( $c = 0; $c < $colsize; $c++ ) {
			// initialise the col as an array...
			$this->cell[ $colnames[ $c ] ] = array();
			for ( $r = 0; $r < $rowsize; $r++ ) {
				// ...and create the cellobjects
				$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ] = &pc4p_create_object( $dummy );
				$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ]->borders = array( "top" => NULL, "bottom" => NULL, "left" => NULL, "right" => NULL );

				// Create and initialize the cellborders
				if ( $r == 0 ) {
					$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ]->borders[ "top" ] = &pc4p_create_object( $dummy, "line" );
					$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ]->borders[ "top" ]->linestyle = "none";
				}
				else {
					$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ]->borders[ "top" ] = &$this->cell[ $colnames[ $c ] ][ $rownames[ $r - 1 ] ]->borders[ "bottom" ];
				}
				$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ]->borders[ "bottom" ] = &pc4p_create_object( $dummy, "line" );
				$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ]->borders[ "bottom" ]->linestyle = "none";
				if ( $c == 0 ) {
					$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ]->borders[ "left" ] = &pc4p_create_object( $dummy, "line" );
					$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ]->borders[ "left" ]->linestyle = "none";
					$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ]->borders[ "left" ]->phi = -90;
				}
				else {
					$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ]->borders[ "left" ] = &$this->cell[ $colnames[ $c - 1 ] ][ $rownames[ $r ] ]->borders[ "right" ];
				}
				$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ]->borders[ "right" ] = &pc4p_create_object( $dummy, "line" );
				$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ]->borders[ "right" ]->linestyle = "none";
				$this->cell[ $colnames[ $c ] ][ $rownames[ $r ] ]->borders[ "right" ]->phi = -90;
			}
		}
	}

	/**
	* Adds a row to the table
	*
	* If you have already defined a tablematrix, you can add a row with this function to every column
	*
	* @param	string	$rowname	Name for the new row
	* @access	public
	* @see		pc4p_init::&pc4p_create_object()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_add_tablerow( $rowname = "" )
	{
		// check if rowname is provided
		if ( empty( $rowname ) ) {
			// no, so use index
			$rowname = sizeof( $this->rownames );
		}
		elseif ( in_array( $rowname, $this->rownames ) ) {
			die( "Error: ".get_class( $this )."(".__LINE__.") - Rowname '".$rowname."' is not unique." );
		}
		// add rowname to object-rownames
		$this->rownames[] = $rowname;

		// use this dummy as object for anchoring the cellobject and the borders...
		$dummy = new pc4p_object( $this );
		for ( $c = 0; $c < sizeof( $this->colnames ); $c++ ) {
			// ...and create new row using the dummy
			$this->cell[ $this->colnames[ $c ] ][ $rowname ] = &pc4p_create_object( $dummy );
			$this->cell[ $this->colnames[ $c ] ][ $rowname ]->borders = array( "top" => NULL, "bottom" => NULL, "left" => NULL, "right" => NULL );

			// Create and initialize the cellborders
			if ( sizeof( $this->rownames ) == 1 ) {
				$this->cell[ $this->colnames[ $c ] ][ $rowname ]->borders[ "top" ] = &pc4p_create_object( $dummy, "line" );
				$this->cell[ $this->colnames[ $c ] ][ $rowname ]->borders[ "top" ]->linestyle = "none";
			}
			else {
				$this->cell[ $this->colnames[ $c ] ][ $rowname ]->borders[ "top" ] = &$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ sizeof( $this->rownames ) - 2] ]->borders[ "bottom" ];
			}
			$this->cell[ $this->colnames[ $c ] ][ $rowname ]->borders[ "bottom" ] = &pc4p_create_object( $dummy, "line" );
			$this->cell[ $this->colnames[ $c ] ][ $rowname ]->borders[ "bottom" ]->linestyle = "none";
			if ( $c == 0 ) {
				$this->cell[ $this->colnames[ $c ] ][ $rowname ]->borders[ "left" ] = &pc4p_create_object( $dummy, "line" );
				$this->cell[ $this->colnames[ $c ] ][ $rowname ]->borders[ "left" ]->linestyle = "none";
				$this->cell[ $this->colnames[ $c ] ][ $rowname ]->borders[ "left" ]->phi = -90;
			}
			else {
				$this->cell[ $this->colnames[ $c ] ][ $rowname ]->borders[ "left" ] = &$this->cell[ $this->colnames[ $c - 1 ] ][ $rowname ]->borders[ "right" ];
			}
			$this->cell[ $this->colnames[ $c ] ][ $rowname ]->borders[ "right" ] = &pc4p_create_object( $dummy, "line" );
			$this->cell[ $this->colnames[ $c ] ][ $rowname ]->borders[ "right" ]->linestyle = "none";
			$this->cell[ $this->colnames[ $c ] ][ $rowname ]->borders[ "right" ]->phi = -90;
		}
	}

	/**
	* Adds a column to the table
	*
	* If you have already defined a tablematrix, you can add a column with this function to every row
	*
	* @param	string	$colname	Name for the new column
	* @access	public
	* @see		pc4p_init::&pc4p_create_object()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_add_tablecol( $colname = "" )
	{
		// check if colname is provided
		if ( empty( $colname ) ) {
			// no, so use index
			$colname = sizeof( $this->colnames );
		}
		elseif ( in_array( $colname, $this->colnames ) ) {
			die( "Error: ".get_class( $this )."(".__LINE__.") - Colname '".$colname."' is not unique." );
		}
		// add rowname to object-colnames
		$this->colnames[] = $colname;

		// use this dummy as object for anchoring the cellobject and the borders...
		$dummy = new pc4p_object( $this );
		for ( $r = 0; $r < sizeof( $this->rownames ); $r++ ) {
			// ...and create new row using the dummy
			$this->cell[ $colname ][ $this->rownames[ $r ] ] = &pc4p_create_object( $dummy );
			$this->cell[ $colname ][ $this->rownames[ $r ] ]->borders = array( "top" => NULL, "bottom" => NULL, "left" => NULL, "right" => NULL );

			// Create and initialize the cellborders
			if ( $r == 0 ) {
				$this->cell[ $colname ][ $this->rownames[ $r ] ]->borders[ "top" ] = &pc4p_create_object( $dummy, "line" );
				$this->cell[ $colname ][ $this->rownames[ $r ] ]->borders[ "top" ]->linestyle = "none";
			}
			else {
				$this->cell[ $colname ][ $this->rownames[ $r ] ]->borders[ "top" ] = &$this->cell[ $colname ][ $this->rownames[ $r - 1 ] ]->borders[ "bottom" ];
			}
			$this->cell[ $colname ][ $this->rownames[ $r ] ]->borders[ "bottom" ] = &pc4p_create_object( $dummy, "line" );
			$this->cell[ $colname ][ $this->rownames[ $r ] ]->borders[ "bottom" ]->linestyle = "none";
			if ( sizeof( $this->colnames ) == 1 ) {
				$this->cell[ $colname ][ $this->rownames[ $r ] ]->borders[ "left" ] = &pc4p_create_object( $dummy, "line" );
				$this->cell[ $colname ][ $this->rownames[ $r ] ]->borders[ "left" ]->linestyle = "none";
				$this->cell[ $colname ][ $this->rownames[ $r ] ]->borders[ "left" ]->phi = -90;
			}
			else {
				$this->cell[ $colname ][ $this->rownames[ $r ] ]->borders[ "left" ] = &$this->cell[ $this->colnames[ sizeof( $this->colnames ) - 2 ] ][ $this->rownames[ $r ] ]->borders[ "right" ];
			}
			$this->cell[ $colname ][ $this->rownames[ $r ] ]->borders[ "right" ] = &pc4p_create_object( $dummy, "line" );
			$this->cell[ $colname ][ $this->rownames[ $r ] ]->borders[ "right" ]->linestyle = "none";
			$this->cell[ $colname ][ $this->rownames[ $r ] ]->borders[ "right" ]->phi = -90;
		}
	}

	/**
	* Sets the width for the columns
	*
	* Takes the provided widths and checks, if you used absolute or percentual sizes
	*
	* @param	array	$colsize	Width for every column. Arraysize has to correspond with the number of columns
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_tablecolsize( $colsize )
	{
		$this->colsize = array();
		if ( is_array( $colsize ) && sizeof( $colsize ) == sizeof( $this->colnames ) ) {
			for ( $i = 0; $i < sizeof( $colsize ); $i++ ) {
				$this->colsize[] = $colsize[ $i ];
			}
		}
		else {
			die( "Error: ".get_class( $this )."(".__LINE__.") - size of columnsizes is not correct." );
		}
	}

	/**
	* Set the style and width of a cellborder
	*
	* With this function you can set the borders of a given cell for each side individually, or completely
	*
	* @param	object	pc4p_object	&$cell	Cell, where you'd like to set the borders
	* @param	mixed	$border	"top", "bottom", "left", "right" or an array of these or "all"
	* @param	mixed	$style	"none", "single" or "double" or an array of these
	* @param	mixed	$width	The width of the line or an array with widths
	* @access	public
	* @see		pc4p_line::pc4p_set_linestyle(), pc4p_line::pc4p_set_linewidth()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_cellborder( &$cell, $border = "all", $style = "single", $width = 1.0 ) {
		// First check which borders will get changed
		if ( !is_array( $border ) && in_array( $border, array( "top", "bottom", "left", "right" ) ) ) {
			$border = array( $border );
		}
		elseif ( $border == "all" ) {
			$border = array( "top", "bottom", "left", "right" );
		}
		// Then pad the style and width to arrays of same size as border, in case they aren't an array
		if ( !is_array( $style ) && in_array( $style, array( "none", "single", "double" ) ) ) {
			$style = array_pad( array(), sizeof( $border ), $style );
		}
		if ( !is_array( $width ) && is_numeric( $width ) ) {
			$width = array_pad( array(), sizeof( $border ), $width );
		}

		// Check for same size of border, style and width
		if ( sizeof( $border ) != sizeof( $style ) || sizeof( $border ) != sizeof( $width ) ) {
			die( "Error: ".get_class( $this )."(".__LINE__.") - when you set the borders of the cell, the size of \$border, \$style and \$width should be the same." );
		}

		// Then set the new parameters for the borders
		for ( $i = 0; $i < sizeof( $border ); $i++ ) {
			$cell->borders[ $border[ $i ] ]->pc4p_set_linestyle( $style[ $i ] );
			$cell->borders[ $border[ $i ] ]->pc4p_set_linewidth( $width[ $i ] );
		}
	}

	/**
	* Set the style of the tableborder
	*
	* With this function you can set the borders of a whole table.
	*
	* @param	mixed	$border	"top", "bottom", "left", "right" or an array of these or all
	* @param	mixed	$style	"none", "single" or "double" or an array of these
	* @param	mixed	$width	The width of the line or an array with widths
	* @access	public
	* @see		pc4p_table::pc4p_set_cellborder()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_tableborder( $border = "all", $style = "single", $width = 1.0 ) {
		$rows = sizeof( $this->rownames );
		$cols = sizeof( $this->colnames );
		for ( $r = 0; $r < $rows; $r++ ) {
			for ( $c = 0; $c < $cols; $c++ ) {
				$this->pc4p_set_cellborder( $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ], $border, $style, $width );
			}
		}
	}

	/**
	* Set the color of a cellborder
	*
	* With this function you can set the bordercolor of a given cell for each side individually, or completely
	*
	* @param	object	pc4p_object	&$cell	Cell, where you'd like to set the bordercolor
	* @param	mixed	$border	"top", "bottom", "left", "right" or an array of these or all
	* @param	mixed	$color	Accepted values are the 16 standardcolors from HTML, the hex-notation (#rrggbb) or an array of these
	* @access	public
	* @see		pc4p_line::pc4p_set_linecolor()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_cellbordercolor( &$cell, $border = "all", $color = "" ) {
		// First check which borders will get changed
		if ( !is_array( $border ) && in_array( $border, array( "top", "bottom", "left", "right" ) ) ) {
			$border = array( $border );
		}
		elseif ( $border == "all" ) {
			$border = array( "top", "bottom", "left", "right" );
		}
		// Then pad the color to an array of same size as border, in case it isn't an array
		if ( !is_array( $color ) && $color != "" ) {
			$color = array_pad( array(), sizeof( $border ), $color );
		}

		// Check for same size of border and color
		if ( sizeof( $border ) != sizeof( $color ) ) {
			die( "Error: ".get_class( $this )."(".__LINE__.") - when you set the bordercolors of the cell, the size of \$border and \$color should be the same." );
		}

		// Then set the new color for the borders
		for ( $i = 0; $i < sizeof( $border ); $i++ ) {
			$cell->borders[ $border[ $i ] ]->pc4p_set_linecolor( $color[ $i ] );
		}
	}

	/**
	* Set the color of the tableborder
	*
	* With this function you can set the bordercolor of a whole table.
	*
	* @param	mixed   $border "top", "bottom", "left", "right" or an array of these or all
	* @param	mixed	$color	Accepted values are the 16 standardcolors from HTML, the hex-notation (#rrggbb) or an array ( "top" => color, "bottom" => color, "left" => color, "right" => color ) where 'color' is one of the forementioned
	* @access	public
	* @see		pc4p_table::pc4p_set_cellbordercolor()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_tablebordercolor( $border = "all", $color = "" ) {
		$rows = sizeof( $this->rownames );
		$cols = sizeof( $this->colnames );
		for ( $r = 0; $r < $rows; $r++ ) {
			for ( $c = 0; $c < $cols; $c++ ) {
				$this->pc4p_set_cellbordercolor( $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ], $border, $color );
			}
		}
	}

	/**
	* Set the fillcolor of a cell
	*
	* With this function you can set the fillcolor of a given cell
	*
	* @param	object	pc4p_object	&$cell	Cell, where you'd like to set the fillcolor
	* @param	mixed	$color	Accepted values are the 16 standardcolors from HTML or the hex-notation (#rrggbb)
	* @access	public
	* @see		pc4p_main::pc4p_check_color()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_cellfillcolor( &$cell, $color ) {
		$cell->cellfillcolor = pc4p_main::pc4p_check_color( $color, "fill" );
	}

	/**
	* Set the fillcolor of the table
	*
	* With this function you can set the fillcolor of a whole table.
	*
	* @param	mixed	$color	Accepted values are the 16 standardcolors from HTML or the hex-notation (#rrggbb)
	* @access	public
	* @see		pc4p_table::pc4p_set_cellfillcolor()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_tablefillcolor( $color ) {
		$rows = sizeof( $this->rownames );
		$cols = sizeof( $this->colnames );
		for ( $r = 0; $r < $rows; $r++ ) {
			for ( $c = 0; $c < $cols; $c++ ) {
				$this->pc4p_set_cellfillcolor( $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ], $color );
			}
		}
	}

	/**
	* Set the header of the table
	*
	* If you want to use a certain row as a header, which is repeated on every new page after a pagebreak within a table, you can supply a rowname here to set it
	*
	* @param	string	$rowname	Name of the row, which shall be used as header
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_tableheader( $rowname ) {
		if ( in_array( $rowname, $this->rownames ) ) {
			$this->tableheader = $rowname;
		}
	}

	/**
	* Splits a table at a given row and returns the rest
	*
	* Very important function for pagebreak
	*
	* @param	integer	$row	Row, where the table shall be split
	* @return	object	pc4p_table	$split	Rest of the table, including the row you specified
	* @access	public
	* @see		pc4p_object::pc4p_make_objcopy()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_split_table( $row )
	{
		$split = pc4p_object::pc4p_make_objcopy( $this );

		$cols = sizeof( $this->colnames );
		$rows = sizeof( $this->rownames );

		// Copy rownames
		$old_rownames = array();
		$new_rownames = array();
		if ( isset( $this->tableheader ) ) {
			$new_rownames[] = $this->tableheader;
		}
		$counter = 0;
		foreach ( $this->rownames as $value ) {
			if ( $counter < $row ) {
				$old_rownames[] = $value;
			}
			else {
				$new_rownames[]	= $value;
			}
			$counter++;
		}

		//Copy columns and rows
		$old_cell = array();
		$new_cell = array();
		foreach ( $this->cell as $ckey => $cvalue ) {
			$counter = 0;
			$old_cell[ $ckey ] = array();
			$new_cell[ $ckey ] = array();
			// This is kinda kludge, but it's the only way to be sure,
			// that we really have a copy and not a copied reference!
			if ( isset( $this->tableheader ) ) {
				$new_cell[ $ckey ][ $this->tableheader ] = pc4p_object::pc4p_make_objcopy( $this->cell[ $ckey ][ $this->tableheader ] );
			}
			foreach ( $cvalue as $rkey => $rvalue ) {
				if ( $counter < $row ) {
					$old_cell[ $ckey ][ $rkey ] = $rvalue;
				}
				else {
					$new_cell[ $ckey ][ $rkey ] = $rvalue;
				}
				$counter++;
			}
		}

		$this->rownames = $old_rownames;
		$split->rownames = $new_rownames;
		$this->cell = $old_cell;
		$split->cell = $new_cell;

		return $split;
	}

	/**
	* Draw-children-function for the table
	*
	* Walks through the rows and cols and calls pc4p_draw in each cell; draws the borders around the cells
	*
	* @access	private
	* @see		pc4p_line::pc4p_get_linewidth(), pc4p_object::pc4p_draw()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_draw_children()
	{
		// Get top coordinate for row
		$rows = sizeof( $this->rownames );
		$cols = sizeof( $this->colnames );
		$this->draw_y = $this->act_y + $this->margin[ "top" ];
		$draw_x = $this->draw_x;
		$draw_y = $this->draw_y;
		for ( $r = 0; $r < $rows; $r++ ) {
			// Get left coordinate for col
			for ( $c = 0; $c < $cols; $c++ ) {
				if( isset( $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->cellfillcolor ) && is_array( $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->cellfillcolor ) ) {
					// First we have to determine the borders of this cell, so that we don't draw the fill into them
					$top_bordersize = $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->borders[ "top" ]->pc4p_get_linewidth();
					$bottom_bordersize = $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->borders[ "bottom" ]->pc4p_get_linewidth();
					if ( $r > 0 ) {
						$top_bordersize /= 2.0;
					}
					if ( $r < $rows - 1 ) {
						$bottom_bordersize /= 2.0;
					}
					$left_bordersize = $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->borders[ "left" ]->pc4p_get_linewidth();
					$right_bordersize = $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->borders[ "right" ]->pc4p_get_linewidth();
					if ( $c > 0 ) {
						$left_bordersize /= 2.0;
					}
					if ( $c < $cols - 1 ) {
						$right_bordersize /= 2.0;
					}
					// Draw a filled rectangle
					pdf_save( $this->pdfp );
					pdf_setcolor( $this->pdfp, "fill", "rgb", $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->cellfillcolor[ 0 ], $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->cellfillcolor[ 1 ], $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->cellfillcolor[ 2 ], 0 );
					pdf_rect( $this->pdfp, $this->draw_x + $left_bordersize, -( $draw_y + $top_bordersize ),  $this->colsize[ $c ] - ( $left_bordersize + $right_bordersize ), -( $this->rowsize[ $r ] - ( $top_bordersize + $bottom_bordersize ) ) );
					pdf_fill( $this->pdfp );
					pdf_restore( $this->pdfp );
				}

				// Call pc4p_draw in the cell
				$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->pc4p_draw();

				// Draw the borders around the cell
				$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->borders[ "top" ]->pc4p_draw();
				if ( $r == $rows - 1 ) {
					$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->borders[ "bottom" ]->pc4p_draw();
				}
				$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->borders[ "left" ]->pc4p_draw();
				if ( $c == $cols - 1 ) {
					$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $r ] ]->borders[ "right" ]->pc4p_draw();
				}

				// Increase coordinate to move to left edge of the next cell
				$this->draw_x += $this->colsize[ $c ];

			}
			$this->draw_x = $draw_x;
			// Increase coordinate to move to top edge of the next row
			$draw_y += $this->rowsize[ $r ];
		}
	}


	/**
	* Calculate the dimensions for a single row and each cell in it
	*
	* Calculates the offsets in all cells, especially the x-coordinates!!! Checks for alignments, predefined columnwidths...
	*
	* @param	object	pc4p_page	&$parent	Parent of the table
	* @param	integer	$row		Which row shall be calculated
	* @return	integer	$row_height	Calculated height of the table
	* @access	private
	* @see		pc4p_line::pc4p_get_linewidth(), pc4p_line::pc4p_calc_linejoin(), pc4p_line::pc4p_set_width(), pc4p_line::pc4p_calc_offset(), pc4p_object::pc4p_set_width(), pc4p_object::pc4p_calc_offset()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_calc_row_offset( &$parent, $row )
	{
		if ( $row == 0 ) {
			if ( get_class( $parent ) == "pc4p_page" ) {
				$this->pagewidth = $parent->width;
				$this->pageheight = $parent->height;
			}
			else {
				$this->pagewidth = $parent->pagewidth;
				$this->pageheight = $parent->pageheight;
			}

			// Check, if someone set the width externally and as a relative value
			if ( isset( $this->width ) && preg_match( "/^(\d+)%$/", $this->width, $width ) ) {
				$this->width = ( $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] ) ) * $width[ 1 ] / 100.0;
			}
			// Check, if width is set, and if yes, if the width exceeds max width defined by parent
			if ( !isset( $this->width ) || $this->width <= 0 || $this->width > $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] ) ) {
				// If-Clause is true, so set width to max width
				$this->width = $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] );
			}

			if ( isset( $this->colsize ) && is_array( $this->colsize ) ) {
				// Calculate the widths accordingly to percentage if used, else
				// just use the absolute value
				$newcolsize = array();
				$emptycols = 0;
				$numcols = sizeof( $this->colsize );
				for ( $i = 0; $i < $numcols; $i++ ) {
					if ( !preg_match( "/^\d+%$/", $this->colsize[ $i ] ) && $this->colsize[ $i ] > 0 ) {
						$newcolsize[ $i ] = $this->colsize[ $i ];
					}
					elseif ( preg_match( "/^(\d+)%$/", $this->colsize[ $i ], $colsize ) ) {
						$newcolsize[ $i ] = ( $this->width - $this->margin[ "left" ] - $this->margin[ "right" ] ) / 100.0 * $colsize[ 1 ];
					}
					else {
						$newcolsize[ $i ] = 0;
						$emptycols++;
					}
				}
				// See, if there's some space left for the cols which weren't set or had
				// a bad parameter. If there isn't any space, well, bad luck.
				if ( $emptycols > 0 && ( $this->width - $this->margin[ "left" ] - $this->margin[ "right" ] ) > array_sum( $newcolsize ) ) {
					$cize = ( $this->width - $this->margin[ "left" ] - $this->margin[ "right" ] - array_sum( $newcolsize ) ) / (float) $emptycols;
					for ( $i = 0; $i < $numcols; $i++ ) {
						if ( $this->colsize[ $i ] <= 0 ) {
							$newcolsize[ $i ] = $cize;
						}
					}
				}
				// Now we take care of the cases, where the colsizes don't equal the width of the table
				// First, we correct the sizes if the sizes are bigger than the table, then vice versa
				if ( array_sum( $newcolsize ) > ( $this->width - $this->margin[ "left" ] - $this->margin[ "right" ] ) ) {
					$factor = array_sum( $newcolsize );
					for ( $i = 0; $i < $numcols; $i++ ) {
						$newcolsize[ $i ] *= ( $this->width - $this->margin[ "left" ] - $this->margin[ "right" ] ) / (float) $factor;
					}
				}
				// We're done *phew*
				$this->colsize = $newcolsize;
			}
			else {
				// No colsizes set, so divide the space evenly between all cols
				$this->colsize = array();
				$numcols = sizeof( $this->colnames );
				$cize = ( $this->width - $this->margin[ "left" ] - $this->margin[ "right" ] ) / (float) $numcols;
				for ( $n = 0; $n < $numcols; $n++) {
					$this->colsize[ $n ] = $cize;
				}
			}

			$this->height = $this->margin[ "top" ];
			$this->act_x = $parent->draw_x;
			$this->act_y = $parent->draw_y;

			// Calculate act_x accordingly to alignment
			if ( $this->alignment == "center" ) {
				$this->draw_x = $this->act_x + $this->margin[ "left" ] + ( $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] ) - $this->width ) / 2;
			}
			elseif ( $this->alignment == "right" ) {
				$this->draw_x = $this->act_x + $this->margin[ "left" ] + $parent->width - ( $parent->margin[ "left" ] + $parent->margin[ "right" ] ) - $this->width;
			}
			else {
				$this->draw_x = $this->act_x + $this->margin[ "left" ];
			}

			$this->draw_y = $this->act_y + $this->margin[ "top" ];

			$this->rowsize = array();
		}

		// Calculate the heights in the cells and add them to our height
		// Set the widths for the cells
		$rows = sizeof( $this->rownames );
		$cols = sizeof( $this->colnames );
		$draw_x = $this->draw_x;
		$draw_y = $this->draw_y;

		// Now we have to determine the maximum top and bottom bordersizes of this row, as we
		// have to add these max-sizes to each cell in this row
		$top_max_bordersize = 0.0;
		$bottom_max_bordersize = 0.0;
		for ( $c = 0; $c < $cols; $c++ ) {
			if ( ( $top_bordersize = $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "top" ]->pc4p_get_linewidth() ) > $top_max_bordersize ) {
				$top_max_bordersize = $top_bordersize;
			}
			if ( ( $bottom_bordersize = $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "bottom" ]->pc4p_get_linewidth() ) > $bottom_max_bordersize ) {
				$bottom_max_bordersize = $bottom_bordersize;
			}
		}
		// If we aren't at an edge, the bordering cells get only half of the border
		if ( $row > 0 ) {
			$top_max_bordersize /= 2.0;
		}
		if ( $row < $rows - 1 ) {
			$bottom_max_bordersize /= 2.0;
		}

		$row_height = 0;
		for ( $c = 0; $c < $cols; $c++ ) {
			// Now calculate the sizes of left and right border
			$left_max_bordersize = $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "left" ]->pc4p_get_linewidth();
			$right_max_bordersize = $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "right" ]->pc4p_get_linewidth();
			if ( $c > 0 ) {
				$left_max_bordersize /= 2.0;
			}
			if ( $c < $cols - 1 ) {
				$right_max_bordersize /= 2.0;
			}

			// Add the borders to the margin
			$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->margin[ "top" ] += $top_max_bordersize;
			$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->margin[ "bottom" ] += $bottom_max_bordersize;
			$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->margin[ "left" ] += $left_max_bordersize;
			$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->margin[ "right" ] += $right_max_bordersize;

			// Calculate height of the children with the width of the current cell
			$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->pc4p_set_width( $this->colsize[ $c ] );
			$child_height = $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->pc4p_calc_offset( $this );

			// Remove the borders again
			$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->margin[ "top" ] -= $top_max_bordersize;
			$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->margin[ "bottom" ] -= $bottom_max_bordersize;
			$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->margin[ "left" ] -= $left_max_bordersize;
			$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->margin[ "right" ] -= $right_max_bordersize;

			$this->draw_x += $this->colsize[ $c ];

			// Save max row height
			if ( $child_height > $row_height ) {
				$row_height = $child_height;
			}
		}

		$this->rowsize[] = $row_height;
		$this->height += $row_height;
		$this->draw_x = $draw_x;
		$this->draw_y += $row_height;

		// Allocate dummy object for calculating the borders
		$dummy = new pc4p_object( $this );
		$dummy->pagewidth = $this->pagewidth;
		$dummy->pageheight = $this->pageheight;
		$dummy->draw_x = $draw_x;
		$dummy->draw_y = $draw_y;
		$dummy->height = $row_height;
		$d = 0;

		for( $c = 0; $c < $cols; $c++ ) {
			$dummy->width = $this->colsize[ $c ];

			// If we don't have an edge, we move the border by half its size
			$adjust_top = 0;
			$adjust_left = 0;
			if ( $row > 0 ) {
				$adjust_top = $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "top" ]->pc4p_get_linewidth() / 2.0;
			}
			if ( $c > 0 ) {
				$adjust_left = $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "left" ]->pc4p_get_linewidth() / 2.0;
			}

			// Now calculate the borders for top and left
			$dummy->draw_y -= $adjust_top;
			$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "top" ]->pc4p_set_width( $dummy->width );
			$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "top" ]->pc4p_calc_offset( $dummy );
			$dummy->draw_y += $adjust_top;

			$dummy->draw_x -= $adjust_left;
			$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "left" ]->pc4p_set_width( $dummy->height );
			$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "left" ]->pc4p_calc_offset( $dummy );
			$dummy->draw_x += $adjust_left;

			// If we are in the last row of a table, draw the bottom
			if ( $row == $rows - 1 ) {
				$adjust_bottom = $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "bottom" ]->pc4p_get_linewidth();
				$dummy->draw_y += $this->rowsize[ $row ];
				$dummy->draw_y -= $adjust_bottom;
				$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "bottom" ]->pc4p_set_width( $dummy->width );
				$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "bottom" ]->pc4p_calc_offset( $dummy );
				$dummy->draw_y += $adjust_bottom;
				$dummy->draw_y -= $this->rowsize[ $row ];
			}

			$dummy->draw_x += $this->colsize[ $c ];

			// If we are in the last column of a row, draw the right side
			if ( $c == $cols - 1 ) {
				$adjust_right = $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "right" ]->pc4p_get_linewidth();
				$dummy->draw_x -= $adjust_right;
				$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "right" ]->pc4p_set_width( $dummy->height );
				$this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "right" ]->pc4p_calc_offset( $dummy );
				$dummy->draw_x += $adjust_right;
			}

			// Now determine the linejoins. We have 9 different kinds of linejoins,
			// and we need to check now, which kind we have...
			if ( $row == 0 ) {
				if ( $c == 0 ) {
					pc4p_line::pc4p_calc_linejoin( $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "top" ], $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "left" ], $d, $d );
				}
				else {
					pc4p_line::pc4p_calc_linejoin( $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "top" ], $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "left" ], $this->cell[ $this->colnames[ $c - 1 ] ][ $this->rownames[ $row ] ]->borders[ "top" ], $d );
				}
				if ( $c == $cols - 1 ) {
					pc4p_line::pc4p_calc_linejoin( $d, $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "right" ], $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "top" ], $d );
				}
			}
			else {
				if ( $c == 0 ) {
					pc4p_line::pc4p_calc_linejoin( $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "top" ], $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "left" ], $d, $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row - 1 ] ]->borders[ "left" ] );
				}
				else {
					pc4p_line::pc4p_calc_linejoin( $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "top" ], $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "left" ], $this->cell[ $this->colnames[ $c - 1 ] ][ $this->rownames[ $row ] ]->borders[ "top" ], $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row - 1 ] ]->borders[ "left" ] );
				}
				if ( $c == $cols - 1 ) {
					pc4p_line::pc4p_calc_linejoin( $d, $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "right" ], $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "top" ], $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row -1 ] ]->borders[ "right" ] );
				}
			}
			if ( $row == $rows - 1 ) {
				if ( $c == 0 ) {
					pc4p_line::pc4p_calc_linejoin( $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "bottom" ], $d, $d, $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "left" ] );
				}
				else {
					pc4p_line::pc4p_calc_linejoin( $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "bottom" ], $d, $this->cell[ $this->colnames[ $c - 1 ] ][ $this->rownames[ $row ] ]->borders[ "bottom" ], $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "left" ] );
				}
				if ( $c == $cols - 1 ) {
					pc4p_line::pc4p_calc_linejoin( $d, $d, $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "bottom" ], $this->cell[ $this->colnames[ $c ] ][ $this->rownames[ $row ] ]->borders[ "right" ] );
				}
			}
		}

		return $row_height;
	}

	/**
	* Calculate the dimensions for the whole table and each cell in it
	*
	* Wraps pc4p_calc_row_offset...
	*
	* @param	object	pc4p_page	&$parent	Parent of the table
	* @return	integer	$this->height	Calculated height of the table
	* @access	private
	* @see		pc4p_table::pc4p_calc_row_offset()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_calc_offset( &$parent )
	{
		for ( $row = 0; $row < sizeof( $this->rownames ); $row++ ) {
			$this->pc4p_calc_row_offset( $parent, $row );
		}

		return $this->height;
	}
}
?>
