<?php
/**
* The text-object
*
* Provides text functionality
*
* @author	Alexander Wirtz <alex@pc4p.net>
* @package	pc4p
* @version	$Id: pc4p_text.inc,v 1.64.2.1 2002/08/20 22:30:08 eru Exp $
*/
class pc4p_text extends pc4p_object
{
	/**
	* Margins for this text
	*
	* @var		array	$margin
	* @access	private
	*/
	var $margin = array( "top" => 0, "bottom" => 0, "left" => 0, "right" => 0 );

	/**
	* Weblink for the object
	*
	* @var		array	$weblink
	* @access	private
	*/
	var $weblink = array( "URI" => "", "style" => "solid", "width" => 1, "color" => array( 0, 0, 0 ) );

	/**
	* Fontpointer for this text
	*
	* @var		integer	$fontp
	* @access	private
	*/
	var $fontp;

	/**
	* Fontname for this text
	*
	* @var		string	$fontname
	* @access	private
	*/
	var $fontname = "Times-Roman";

	/**
	* Fontsize for this text
	*
	* @var		integer	$fontsize
	* @access	private
	*/
	var $fontsize = 10;

	/**
	* Underline for this text
	*
	* @var		integer	$underline
	* @access	private
	*/
	var $underline = 0;

	/**
	* Textrendering for this object
	*
	* @var		integer	$rendering
	* @access	private
	*/
	var $rendering = 0;

	/**
	* Text for this text...haha
	*
	* @var		array	$text
	* @access	private
	*/
	var $text;

	/**
	* Textcolor
	*
	* @var		array	$textcolor
	* @access	private
	*/
	var $textcolor = array( 0, 0, 0 );

	/**
	* pc4p_text constructor
	*
	* Calls the constructor of pc4p_object and sets the textleading to the value of the parent...if it's not empty
	*
	* @param	object	pc4p_page	&$parent	Parent of the text
	* @access	public
	* @see		pc4p_object::pc4p_object()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_text( &$parent )
	{
		pc4p_object::pc4p_object( $parent );

		// Let's see, if someone defined the leading
		if ( !empty( $parent->leading ) ) {
			$this->leading = $parent->leading;
		}
	}

	/**
	* Sets a weblink for this object.
	*
	* Add a weblink to this object, that will be opened on clicking on this object
	*
	* @param	string	$weblink	The URI of the weblink
	* @param	string	$style		The style of the border representing the weblink; "solid" or "dashed"
	* @param	integer	$width		The width of the border representing the weblink; 0 represents an invisible border
	* @param	string	$color		The color of the border representing the weblink; you can use one of the 16 standardcolors from HTML or the hex-notation (#rrggbb)
	* @see		pc4p_main::pc4p_check_color()
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_weblink( $weblink, $style = "", $width = -1, $color = "" )
	{
		if( !empty( $weblink ) ) {
			$this->weblink[ "URI" ] = $weblink;
			if( !empty( $style ) && ( $style == "solid" || $style == "dashed" ) ) {
				$this->weblink[ "style" ] = $style;
			}
			if( is_numeric( $width ) && $width >= 0 ) {
				$this->weblink[ "width" ] = $width;
			}
			if( !empty( $color ) ) {
				$this->weblink[ "color" ] = pc4p_main::pc4p_check_color( $color, "line" );
			}
		}
	}

	/**
	* Sets the alignment for the text
	*
	* The same functionality as in every other document-software
	*
	* @param	string	$alignment	Valid values are "left", "center", "right" and "justify"
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_alignment( $alignment = "left" )
	{
		$accepted_values = array( "left", "center", "right", "justify" );
		if ( in_array( strtolower( $alignment ), $accepted_values ) ) {
			$this->alignment = $alignment;
		}
	}

	/**
	* Sets the font for this object.
	*
	* This is the function, with which you define everything concerning the font of your text.
	*
	* @param	string	$fontname	Name of your font
	* @param	integer	$fontsize	Size of the font...reasonable values please
	* @param	string	$encoding	You should leave this alone, until you know what you're doing
	* @param	integer	$embed		Triggers if the font will get embedded
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_font( $fontname, $fontsize = 10, $encoding = "host", $embed = 0 )
	{
		// Check the parameters for correct input
		if ( is_string( $fontname ) ) {
			$this->fontname = $fontname;
		}
		if ( is_numeric( $fontsize ) ) {
			$this->fontsize = $fontsize;
		}
		if ( !in_array( $encoding, array( "host", "builtin", "macroman", "winansi", "ebcdic" ) ) ) {
			$encoding = "host";
		}
		if ( $embed ) {
			$embed = 1;
		}
		else {
			$embed = 0;
		}

		// Generate hash-key for the font-lookup-table
		$key = md5( $this->fontname."-".$encoding );
		if ( function_exists( "array_key_exists" ) ) {
			$key_func = "array_key_exists";
		}
		else {
			$key_func = "key_exists";
		}
		if ( $key_func( $key,  $this->pdfobj->fonts ) ) {
			// Font is already allocated, so use its fontpointer
			$this->fontp = $this->pdfobj->fonts[ $key ];
		}

		if ( !isset( $this->fontp ) ) {
			// Font wasn't in lookup-table, allocate it
			$this->fontp = pdf_findfont( $this->pdfp, $this->fontname, $encoding, $embed );
			if ( !$this->fontp ) {
				die( "Error: ".get_class( $this )."(".__LINE__.") - ".$this->fontname." is not a valid fontname or you chose a wrong encoding." );
			}
			// Put fontpointer into the lookup-table for further usage
			$this->pdfobj->fonts[ $key ] = $this->fontp;
		}
	}

	/**
	* Sets the text which the object shall print later on
	*
	* Do I really have to explain this function? I hope not...
	*
	* @param	string	$text	Accepts a string. Use \n for linefeeds.
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_text( $text )
	{
		// Replace a few escape-sequences with harmless equivalents, so that we then can safely explode the string by \n.
		$trim = array( "\r" => "", "\t" => " ",  "\0" => "",  "\x0B" => "" );
		$text = strtr( $text, $trim );
		$this->text = explode( "\n", $text );
	}

	/**
	* Shall the text have an underline?
	*
	* Call this function to enable underline for your text
	*
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_underline()
	{
		if ( $this->underline == 0 ) {
			$this->underline = 1;
		}
		else {
			$this->underline = 0;
		}
	}

	/**
	* Changes the text-rendering
	*
	* Use this function, if you want to change the rendering-mode of this textobject
	*
	* @param	integer	$mode	We only support modes 0 to 3, as clipping paths make no sense here ( 0 => fill text, 1 => outline text, 2 => fill and outline, 3 => invisible text )
	* @access	public
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_rendering( $mode )
	{
		if ( is_integer( $mode ) && $mode >= 0 && $mode <= 3 ) {
			$this->rendering = $mode;
		}
		else {
			$this->rendering = 0;
		}
	}

	/**
	* Sets the color for this text.
	*
	* You can define a textcolor here or reset the fillcolor to default, if you don't give a parameter
	*
	* @param	string	$textcolor	You can use one of the 16 standardcolors from HTML or the hex-notation (#rrggbb)
	* @access	public
	* @see		pc4p_main::pc4p_check_color()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_set_textcolor( $textcolor )
	{
		$this->textcolor = pc4p_main::pc4p_check_color( $textcolor, "line" );
	}

	/**
	* Splits a text at a given row and returns the rest
	*
	* Very important function for pagebreak
	*
	* @param	integer	$row	Row, where the table shall be split
	* @return	object	pc4p_text	$split	Rest of the text, including the row you specified
	* @access	public
	* @see		pc4p_object::pc4p_make_objcopy()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_split_text( $row )
	{
		$split = pc4p_object::pc4p_make_objcopy( $this );

		$split->text = array_splice( $this->text, $row );

		return $split;
	}

	/**
	* Formats the text
	*
	* This function formats the text for the actual width. Returns the rows which the text needs.
	*
	* @param	string	&$text	Text, which shall be linewrapped
	* @return	integer	$lines	That many lines are needed for the input-string
	* @access	private
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_linefeed( &$text )
	{
		for ( $i = 0; $i < sizeof( $text ); $i++ ) {
			// Check, if the line has the correct width
			if ( pdf_stringwidth( $this->pdfp, $text[ $i ] ) > $this->width ) {
				// Nope...explode the line, so that we have single words
				$line = explode( " ", $text[ $i ] );
				$newtext = array();
				$newline = "";
				foreach ( $line as $word ) {
					// Now check, if the word fits into the line
					if ( pdf_stringwidth( $this->pdfp, $newline." ".$word ) <= $this->width ) {
						$newline .= " ".$word;
					}
					else {
						// Check, if the word is larger than the width
						if ( pdf_stringwidth( $this->pdfp, $word ) <= $this->width ) {
							// Nope, so start a new line
							$newtext[] = trim( $newline );
							$newline = $word;
						}
						else {
							// It is larger, so we have to check for every single letter
							$wordchunk = "";
							if ( strlen( $newline ) > 0 ) {
								$newline .= " ";
							}
							for ( $j = 0; $j < strlen( $word ); $j++ ) {
								// Does the letter fit into the line?
								if ( pdf_stringwidth( $this->pdfp, $newline.$wordchunk.$word{ $j } ) <= $this->width ) {
									$wordchunk .= $word{ $j };
								}
								else {
									// Nope, start a new line
									$newtext[] = $newline.$wordchunk;
									$newline = "";
									$wordchunk = $word{ $j };
								}
							}
							// Don't forget the rest!
							if ( strlen( $wordchunk ) > 0 ) {
								$newline = $wordchunk;
							}
						}
					}
				}
				// Here too!
				if ( strlen( $newline ) > 0 ) {
					$newtext[] = $newline;
				}
				// Move the splitted line into the textarray
				array_splice( $text, $i, 1, $newtext );
			}
		}
		// and finally, return the lines
		$lines = sizeof( $text );
		return $lines;
	}

	/**
	* Draw-function for the text
	*
	* Draws the text for the set fontoptions and the calculated  linefeeds
	*
	* @access	public
	* @see		pc4p_text::pc4p_calc_alignment()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_draw()
	{
		pdf_save( $this->pdfp );
		pdf_setfont( $this->pdfp, $this->fontp, $this->fontsize );
		pdf_setcolor( $this->pdfp, "fill", "rgb", $this->textcolor[ 0 ], $this->textcolor[ 1 ], $this->textcolor[ 2 ], 0 );
		pdf_setcolor( $this->pdfp, "stroke", "rgb", $this->textcolor[ 0 ], $this->textcolor[ 1 ], $this->textcolor[ 2 ], 0 );
		if ( $this->underline ) {
			pdf_set_parameter( $this->pdfp, "underline", "true" );
		}
		if ( $this->rendering ) {
			pdf_set_value( $this->pdfp, "textrendering", $this->rendering );
		}

		$this->draw_y += $this->fontsize - 2;
		// These vars are used to determine the box around the text for weblinks
		$web_x = array();
		$web_width = array();
		for ( $i = 0; $i < sizeof( $this->text ); $i++ ) {
			if ( strlen( $this->text[ $i ]) > 0 ) {
				$draw_x = $this->pc4p_calc_alignment( $this->text[ $i ] );
				if( $this->alignment == "justify" ) {
					if( ( $spcount = substr_count( $this->text[ $i ], " " ) ) > 0 ) {
						if( ( $wordspacing = ( $this->width - pdf_stringwidth( $this->pdfp, $this->text[ $i ] ) ) / $spcount ) > 0 ) {
							pdf_set_value( $this->pdfp, "wordspacing", $wordspacing );
						}
					}
				}
				pdf_show_xy( $this->pdfp, $this->text[ $i ], $draw_x, -$this->draw_y );
				$this->draw_y += $this->fontsize + ( $this->leading - $this->fontsize );
				$web_x[] = $draw_x;
				$web_width[] = pdf_stringwidth( $this->pdfp, $this->text[ $i ] );
				pdf_set_value( $this->pdfp, "wordspacing", 0 );
			}
			else {
				$this->draw_y += $this->fontsize + ( $this->leading - $this->fontsize );
			}
		}

		// The handling of the pdf_add_weblink-func is crappy, because it doesn't honor a pdf_translate. Everything has to be calculated manually :-(
		if( !empty( $this->weblink[ "URI" ] ) ) {
			pdf_set_border_style( $this->pdfp, $this->weblink[ "style" ], $this->weblink[ "width" ] );
			pdf_set_border_color( $this->pdfp, $this->weblink[ "color" ][ 0 ], $this->weblink[ "color" ][ 1 ], $this->weblink[ "color" ][ 2 ] );
			pdf_add_weblink( $this->pdfp, min( $web_x ), $this->pageheight - $this->act_y, min( $web_x ) + max( $web_width ), $this->pageheight - $this->act_y - $this->height, $this->weblink[ "URI" ] );
		}

		if ( $this->underline ) {
			pdf_set_parameter( $this->pdfp, "underline", "false" );
		}
		if ( $this->rendering ) {
			pdf_set_value( $this->pdfp, "textrendering", 0 );
		}
		pdf_restore( $this->pdfp );

		unset( $this->pdfobj );
	}

	/**
	* Calculates the text-alignment
	*
	* The position of the text according to the set alignment is calculated.
	*
	* @param	string	$text	The text...no further explanation needed
	* @return	integer	$draw_x	The position according to the alignment
	* @access	private
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_calc_alignment( $text )
	{
		$strwidth = pdf_stringwidth( $this->pdfp, $text );
		if ( $this->alignment == "center" ) {
			$draw_x = $this->draw_x + ( $this->width - $strwidth ) / 2;
		}
		elseif ( $this->alignment == "right" ) {
			$draw_x = $this->draw_x + $this->width - $strwidth;
		}
		else {
			$draw_x = $this->draw_x;
		}
		return $draw_x;
	}

	/**
	* Calculates the dimensions for the text
	*
	* Calculates the stringwidth for the current text and font in the
	* object and applies linefeed where necessary. Returns the size
	* for the complete object.
	*
	* @param	object	pc4p_page	&$parent	Parent of the text
	* @return	int		$this->height	Calculated height of the text
	* @access	private
	* @see		pc4p_text::pc4p_linefeed()
	* @author	Alexander Wirtz <alex@pc4p.net>
	*/
	function pc4p_calc_offset( &$parent )
	{
		if ( get_class( $parent ) == "pc4p_page" ) {
			$this->pagewidth = $parent->width;
			$this->pageheight = $parent->height;
		}
		else {
			$this->pagewidth = $parent->pagewidth;
			$this->pageheight = $parent->pageheight;
		}

		if ( empty( $this->leading ) || $this->leading < $this->fontsize ) {
			$this->leading = $this->fontsize;
		}

		if ( !isset( $this->fontp ) ) {
			$this->pc4p_set_font( $this->fontname );
		}
		pdf_setfont( $this->pdfp, $this->fontp, $this->fontsize );

		$this->act_x = $parent->draw_x;
		$this->act_y = $parent->draw_y;

		$this->draw_x = $this->act_x + $this->margin[ "left" ];
		$this->draw_y = $this->act_y + $this->margin[ "top" ];

		$this->width = $parent->width - ( $this->margin[ "left" ] + $this->margin[ "right" ] + $parent->margin[ "left" ] + $parent->margin[ "right" ] );

		$lines = $this->pc4p_linefeed( $this->text );

		$this->height = $this->margin[ "top" ] + ( ( $this->fontsize + ( $this->leading - $this->fontsize ) ) * $lines ) + $this->margin[ "bottom" ];

		return $this->height;
	}
}
?>
